//+------------------------------------------------------------------+
//|                   HMA Color nrp & mtf & alerts + arrows 2.02.mq4 |
//|                                                           mladen |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link "mladenfx@gmail.com"

#property indicator_chart_window
#property indicator_buffers 7
#property indicator_color1  Orange//Yellow
#property indicator_color2  Lime//LimeGreen
#property indicator_color3  Lime//LimeGreen
#property indicator_color4  Crimson//Orange
#property indicator_color5  Crimson//Orange
#property indicator_color6  DodgerBlue
#property indicator_color7  Magenta
#property indicator_width1  4
#property indicator_width2  4
#property indicator_width3  4
#property indicator_width4  4
#property indicator_width5  4

extern string TimeFrame        = "Current time frame";
extern int    HMA_Period       = 35;
extern int    HMA_PriceType    = PRICE_CLOSE;
extern int    HMA_Method       = 3;
extern bool   DisplayHmaValue  = true;
extern bool   alertsOn         = true;
extern bool   alertsOnCurrent  = false;
extern bool   alertsMessage    = true;
extern bool   alertsSound      = true;
extern bool   alertsEmail      = false;
extern bool   Interpolate      = true;

extern bool   ShowArrows       = true;
extern int    arrowSize        = 2;
extern int    uparrowCode      = 233;
extern int    dnarrowCode      = 234;
extern bool   ArrowsOnFirstBar = false;//true;

int HalfPeriod;
int HullPeriod;

double ind_buffer0[];
double ind_buffer1[];
double ind_buffer2[];
double ind_buffer3[];
double ind_buffer4[];
double arrowUp[];
double arrowDn[];
double buffer[];
double trend[];

string indicatorFileName;
bool   returnBars;
int    timeFrame;

//+------------------------------------------------------------------
//|                                                                 |
//+------------------------------------------------------------------

int init()
{
   HMA_Period = MathMax(2,HMA_Period);
   HalfPeriod = MathFloor(HMA_Period/2);
   HullPeriod = MathFloor(MathSqrt(HMA_Period));
  
   IndicatorBuffers(9);
   SetIndexBuffer(0,ind_buffer0);
   SetIndexBuffer(1,ind_buffer1);
   SetIndexBuffer(2,ind_buffer2);
   SetIndexBuffer(3,ind_buffer3);
   SetIndexBuffer(4,ind_buffer4);
   SetIndexBuffer(5,arrowUp);   
   SetIndexBuffer(6,arrowDn);  
   if (DisplayHmaValue)
         for (int i=0; i<7; i++) SetIndexStyle(i,DRAW_LINE);
   else  for (    i=0; i<7; i++) SetIndexStyle(i,DRAW_NONE);
   SetIndexBuffer(7,buffer);
   SetIndexBuffer(8,trend);
   if (ShowArrows)
   {
     SetIndexStyle(5,DRAW_ARROW,0,arrowSize); SetIndexArrow(5,uparrowCode);
     SetIndexStyle(6,DRAW_ARROW,0,arrowSize); SetIndexArrow(6,dnarrowCode);   
   }
   else
   {
     SetIndexStyle(5,DRAW_NONE);
     SetIndexStyle(6,DRAW_NONE);
   }       

      indicatorFileName = WindowExpertName();
      returnBars        = (TimeFrame=="returnBars");     if (returnBars)     return(0);
      timeFrame         = stringToTimeFrame(TimeFrame);

   int draw_begin=HMA_Period+MathFloor(MathSqrt(HMA_Period));
   for (i = 0; i < indicator_buffers; i++)
   {
         SetIndexDrawBegin(i,draw_begin);
         SetIndexLabel(i,"Hull Moving Average");
   }         
   IndicatorShortName("HMA("+HMA_Period+","+TimeFrame+")");
   return(0);
}

int deinit(){ return(0); }

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+

int start()
{
   int counted_bars = IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
           int limit = MathMin(Bars-counted_bars,Bars-1);
           if (returnBars) { ind_buffer0[0] = limit+1; return(0); }
           if (timeFrame != Period())
           {
               int shift = -1; if (ArrowsOnFirstBar) shift=1;
               limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
               if (trend[limit] ==  1) CleanPoint(limit,ind_buffer1,ind_buffer2);
               if (trend[limit] == -1) CleanPoint(limit,ind_buffer3,ind_buffer4);
               for(int i=limit; i>=0; i--)
               {
                  int y = iBarShift(NULL,timeFrame,Time[i]);
                  int x = iBarShift(NULL,timeFrame,Time[i+shift]);
                     ind_buffer0[i] = iCustom(NULL,timeFrame,indicatorFileName,"CalculateValue",HMA_Period,HMA_PriceType,HMA_Method,DisplayHmaValue,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,0,y);
                     ind_buffer1[i] = EMPTY_VALUE;
                     ind_buffer2[i] = EMPTY_VALUE;
                     ind_buffer3[i] = EMPTY_VALUE;
                     ind_buffer4[i] = EMPTY_VALUE;
                     trend[i]       = iCustom(NULL,timeFrame,indicatorFileName,"CalculateValue",HMA_Period,HMA_PriceType,HMA_Method,DisplayHmaValue,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,8,y);
                  if(x!=y)
                  {
                     arrowUp[i]     = iCustom(NULL,timeFrame,indicatorFileName,"CalculateValue",HMA_Period,HMA_PriceType,HMA_Method,DisplayHmaValue,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,5,y);
                     arrowDn[i]     = iCustom(NULL,timeFrame,indicatorFileName,"CalculateValue",HMA_Period,HMA_PriceType,HMA_Method,DisplayHmaValue,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,6,y);
                  }
                  else
                  {
                     arrowUp[i]     = EMPTY_VALUE;
                     arrowDn[i]     = EMPTY_VALUE;
                  }
         
                      if (!Interpolate || y==iBarShift(NULL,timeFrame,Time[i-1])) continue;

                      datetime time = iTime(NULL,timeFrame,y);
                        for(int n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;	
                        for(int k = 1; k < n; k++) ind_buffer0[i+k] = ind_buffer0[i] + (ind_buffer0[i+n] - ind_buffer0[i])* k/n;                             
               }
               for(i=limit; i>=0; i--)
               {
                  if (trend[i] ==  1) PlotPoint(i,ind_buffer1,ind_buffer2,ind_buffer0);
                  if (trend[i] == -1) PlotPoint(i,ind_buffer3,ind_buffer4,ind_buffer0);
               }
               return(0);     
           }
  
   if (trend[limit] ==  1) CleanPoint(limit,ind_buffer1,ind_buffer2);
   if (trend[limit] == -1) CleanPoint(limit,ind_buffer3,ind_buffer4);
   for(i=limit; i>=0; i--)
         buffer[i]=iMA(NULL,0,HalfPeriod,0,HMA_Method,HMA_PriceType,i)*2-
                   iMA(NULL,0,HMA_Period,0,HMA_Method,HMA_PriceType,i);
   for(i=limit; i>=0; i--)
   {
      ind_buffer0[i] = iMAOnArray(buffer,0,HullPeriod,0,HMA_Method,i);
      ind_buffer1[i] = EMPTY_VALUE;
      ind_buffer2[i] = EMPTY_VALUE;
      ind_buffer3[i] = EMPTY_VALUE;
      ind_buffer4[i] = EMPTY_VALUE;

      trend[i] = trend[i+1];            
         if (ind_buffer0[i] > ind_buffer0[i+1]) trend[i] =  1; 
         if (ind_buffer0[i] < ind_buffer0[i+1]) trend[i] = -1; 
         if (trend[i] ==  1) PlotPoint(i,ind_buffer1,ind_buffer2,ind_buffer0);
         if (trend[i] == -1) PlotPoint(i,ind_buffer3,ind_buffer4,ind_buffer0);
    
         arrowUp[i] = EMPTY_VALUE;
         arrowDn[i] = EMPTY_VALUE;
         if (trend[i]!= trend[i+1])
         if (trend[i] == 1)
               arrowUp[i] = ind_buffer0[i] - iATR(NULL,0,20,i)/2.0;
         else  arrowDn[i] = ind_buffer0[i] + iATR(NULL,0,20,i)/2.0;
   }
   manageAlerts();
   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+

void CleanPoint(int i,double& first[],double& second[])
{
   if ((second[i]  != EMPTY_VALUE) && (second[i+1] != EMPTY_VALUE))
        second[i+1] = EMPTY_VALUE;
   else
      if ((first[i] != EMPTY_VALUE) && (first[i+1] != EMPTY_VALUE) && (first[i+2] == EMPTY_VALUE))
          first[i+1] = EMPTY_VALUE;
}

void PlotPoint(int i,double& first[],double& second[],double& from[])
{
   if (first[i+1] == EMPTY_VALUE)
      {
      if (first[i+2] == EMPTY_VALUE) {
          first[i]    = from[i];
          first[i+1]  = from[i+1];
          second[i]   = EMPTY_VALUE;
         }
      else {
          second[i]   = from[i];
          second[i+1] = from[i+1];
          first[i]    = EMPTY_VALUE;
         }
      }
   else
      {
         first[i]   = from[i];
         second[i]  = EMPTY_VALUE;
      }
}

//-------------------------------------------------------------------
//                                                                  
//-------------------------------------------------------------------

void manageAlerts()
{
   if (alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1; 
      if (trend[whichBar] != trend[whichBar+1])
      {
         if (trend[whichBar] ==  1) doAlert(whichBar,"up");
         if (trend[whichBar] == -1) doAlert(whichBar,"down");
      }
   }
}

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[forBar]) {
       previousAlert  = doWhat;
       previousTime   = Time[forBar];

       message =  timeFrameToString(Period())+" "+Symbol()+" at "+TimeToStr(TimeLocal(),TIME_SECONDS)+" hull trend changed to "+doWhat;
          if (alertsMessage) Alert(message);
          if (alertsEmail)   SendMail(Symbol()+" hull trend",message);
          if (alertsSound)   PlaySound("alert2.wav");
   }
}

//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------

string sTfTable[] = {"M1","M5","M10","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,10,15,30,60,240,1440,10080,43200};

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int tchar = StringGetChar(s, length);
         if((tchar > 96 && tchar < 123) || (tchar > 223 && tchar < 256))
                     s = StringSetChar(s, length, tchar - 32);
         else if(tchar > -33 && tchar < 0)
                     s = StringSetChar(s, length, tchar + 224);
   }
   return(s);
}